#include <stdio.h>
#include <string.h>

// RC server
#include <arpa/inet.h>        /*  inet (3) funtions         */
#include <errno.h>

/*  Global constants  */
#define SERVER_PORT          (2345)

ssize_t Readline(int sockd, void *vptr, size_t maxlen);

// signal strength check mod 20/02/2011
// use: netcat (etc.), port 2346
//  or: send "AD" keycode to 2345 port && check /mtd_ram/SignalStrength.txt file
extern unsigned int _ZN8SsObject14m_poObjectListE[];
extern int _ZN14SsFrontEndBase17GetSignalStrengthEPhb(unsigned int, int *, int);
extern int _ZN14SsFrontEndBase16GetSignalQualityEPh(unsigned int, int *);

int chkSiSt() {
   int sval=0;
   _ZN14SsFrontEndBase17GetSignalStrengthEPhb(_ZN8SsObject14m_poObjectListE[7], &sval, 0);
   return(sval);
}

int chkSiQa() {
   int sval=0;
   _ZN14SsFrontEndBase16GetSignalQualityEPh(_ZN8SsObject14m_poObjectListE[7], &sval);
   return(sval);
}

void writeSiSt(int val, int q) {
   FILE *fh=fopen( "/mtd_ram/SignalStrength.txt", "w+" );
   if (fh == NULL) return;
   fprintf(fh, "%d %d\n", val, q);
   fclose(fh);
}

void SiSt_thread(void *v)
{
	FILE *hlog;
	hlog = fopen( "/mtd_ram/remote_SiSt.log", "a+" );
	fprintf( hlog, "SiSt_thread started: arg - 0x%08X\n", v );
	fflush(hlog);
	
	int       list_s;
	int       conn_s;
	short int port;  
	struct    sockaddr_in servaddr;
	char      buffer[100];

	if ( (list_s = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		fprintf(hlog, "Error creating listening socket.\n");
		fclose(hlog);
		return;
    	}

	int on = 1;
	setsockopt(list_s, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(int));

	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family      = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(SERVER_PORT+1);

	if ( bind(list_s, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 ) 
	{
		fprintf(hlog, "Error calling bind()\n");
		fclose(hlog);
		return;
	}

	if ( listen(list_s, 10) < 0 ) 
	{
		fprintf(hlog, "Error calling listen()\n");
		fclose(hlog);
		return;
	}

	while (1)
	{
		if ( (conn_s = accept(list_s, NULL, NULL) ) < 0 ) {
		    fprintf( hlog, "Error calling accept()\n");
		    break;
		}

      int sval=chkSiSt();
      int qval=chkSiQa();
      writeSiSt(sval, qval);
      
		memset(buffer, 0, 100);
		sprintf(buffer, "%d %d\n", sval, qval);

		fprintf(hlog, "Signal strength: %d   quality: %d\n", sval, qval);
		fflush(hlog);
   	write(conn_s, buffer, strlen(buffer));

		if ( close(conn_s) < 0 ) 
		{
		    fprintf(hlog, "Error calling close()\n");
		    break;
		}
	}

	fclose(hlog);
	close(list_s);
	return;
}

//

void remote_thread( void *v )
{
	FILE *hlog;

	hlog = fopen( "/mtd_ram/remote.log", "a+" );

	fprintf( hlog, "remote_thread started: pInstance - 0x%08X\n", v );
	fflush(hlog);
	
	int       list_s;
	int       conn_s;
	short int port;  
	struct    sockaddr_in servaddr;
	char      buffer[1024]; 
	char     *endptr;    
	int       t2arg=0;
	pthread_t helper2;

	if ( (list_s = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		fprintf(hlog, "Error creating listening socket.\n");
		fclose(hlog);
		return;
    	}

	int on = 1;
	setsockopt(list_s, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(int));

	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family      = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(SERVER_PORT);

	if ( bind(list_s, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 ) 
	{
		fprintf(hlog, "Error calling bind()\n");
		fclose(hlog);
		return;
	}

	if ( listen(list_s, 10) < 0 ) 
	{
		fprintf(hlog, "Error calling listen()\n");
		fclose(hlog);
		return;
	}

   // mod
	pthread_create( &helper2, NULL, (void *)SiSt_thread, &t2arg);

	while (1)
	{
		if ( (conn_s = accept(list_s, NULL, NULL) ) < 0 ) {
		    fprintf( hlog, "ECHOSERV: Error calling accept()\n");
		    break;
		}

		memset( buffer, 0, 1024 );
		Readline(conn_s, buffer, 1023);

		if ( buffer[0] == 'Q' )
		{
		    fprintf( hlog, "exit..\n");
			close(conn_s);
			break;
		}

		int key = atoi( buffer );
		
		// mod
		if (key == 0x27) writeSiSt(chkSiSt(), chkSiQa());

		fprintf( hlog, "received key: %d\n", key );
		fflush(hlog);

		asm volatile(
			"MOV	R0,%0\n"
			"MOV 	R1,%1\n"
			"BL	_ZN14SsKeyInputBase7SendKeyEi\n"
			:
			: "r" (v), "r" (key) );

		if ( close(conn_s) < 0 ) 
		{
		    fprintf(hlog, "Error calling close()\n");
		    break;
		}
	}

	fclose(hlog);
	close(list_s);
	return;
}

ssize_t Readline(int sockd, void *vptr, size_t maxlen) {
    ssize_t n, rc;
    char    c, *buffer;

    buffer = vptr;

    for ( n = 1; n < maxlen; n++ ) {
	
	if ( (rc = read(sockd, &c, 1)) == 1 ) {
	    *buffer++ = c;
	    if ( c == '\n' )
		break;
	}
	else if ( rc == 0 ) {
	    if ( n == 1 )
		return 0;
	    else
		break;
	}
	else {
	    if ( errno == EINTR )
		continue;
	    return -1;
	}
    }

    *buffer = 0;
    return n;
}

